package dev.fitko.fitconnect.integrationtests;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.model.callback.NewSubmissionsCallback;
import dev.fitko.fitconnect.api.domain.validation.ValidationResult;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.bootstrap.ClientFactory;
import dev.fitko.fitconnect.client.sender.model.SendableSubmission;
import dev.fitko.fitconnect.core.http.HttpClient;
import dev.fitko.fitconnect.integrationtests.condition.EnableIfEnvironmentVariablesAreSet;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.net.URI;
import java.util.List;
import java.util.UUID;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

@EnableIfEnvironmentVariablesAreSet
public class CallbackIT extends IntegrationTestBase {

    private final String callbackRequestUrl = "https://webhook.site/token/14efa049-f720-45d5-8088-ecbff3f625dd/request/latest";
    private final ObjectMapper objectMapper = new ObjectMapper().configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);


    @Test
    void testSubmissionCallbackNotification() throws IOException, InterruptedException {

        // Given
        final HttpClient httpClient = new HttpClient();
        final ApplicationConfig config = getConfigWithCredentialsFromEnvironment("TESTING", true);
        final SenderClient senderClient = ClientFactory.getSenderClient(config);
        final String submissionData = getResourceAsString("/submission_data.json");

        final var submission = SendableSubmission.Builder()
                .setDestination(UUID.fromString(System.getenv("TEST_DESTINATION_ID_JSON_DATA")))
                .setServiceType("urn:de:fim:leika:leistung:99400048079000", "Test Service")
                .setJsonData(submissionData, URI.create("https://schema.fitko.de/fim/s00000114_1.1.schema.json"))
                .build();

        final var sentSubmission = senderClient.send(submission);

        assertNotNull(sentSubmission);

        // Wait for callback to be retrieved by webhook
        Thread.sleep(5000);

        // When
        final WebhookResponse webhookResponse = httpClient.get(callbackRequestUrl, WebhookResponse.class).getBody();
        final NewSubmissionsCallback callbackBody = objectMapper.readValue(webhookResponse.getContent(), NewSubmissionsCallback.class);

        // Then
        assertThat(callbackBody.getCallbackType(), is("https://schema.fitko.de/fit-connect/submission-api/callbacks/new-submissions"));
        assertThat(callbackBody.getSubmissions(), hasSize(1));
        assertThat(callbackBody.getSubmissions().get(0).getSubmissionId(), is(sentSubmission.getSubmissionId()));

        final String hmac = webhookResponse.getHeaders().getCallbackAuthentication().get(0);
        final Long timestampInSec = webhookResponse.getHeaders().getCallbackTimestamp().get(0);

        final ValidationResult validationResult = senderClient.validateCallback(hmac, timestampInSec, webhookResponse.getContent(), "callbackSecretWithMoreThan32Chars");

        assertTrue(validationResult.isValid());
    }

    @Data
    @NoArgsConstructor
    static class WebhookResponse {
        private String content;
        private Headers headers;

        @Data
        @NoArgsConstructor
        static class Headers {

            @JsonProperty("callback-timestamp")
            private List<Long> callbackTimestamp;

            @JsonProperty("callback-authentication")
            private List<String> callbackAuthentication;
        }
    }

}
